/*-----------------------------------------------------------------------*/
/* Gets synthesis parameters from the lisp structs specified in Scheme   */
/*=======================================================================*/
#include <stdlib.h>
#include "festival.h"
#include "db_alloc.h"
#include "fl_globals.h"

#define DEF_VIBRATO_FREQ 5.5
#define DEF_VIBRATO_MAX  0.05
#define DEF_BEND_SEMI_MAX 12.0
#define DEF_PORTAMENTO_TIME_DEFAULT 0.010
#define DEF_DRIFT_FREQ1 4.7
#define DEF_DRIFT_FREQ2 7.1
#define DEF_DRIFT_FREQ3 12.7
#define DEF_TRANSPOSE 0.0
#define DEF_DRIFT_AMPL 0.002

#define DEF_CONSONANT_STRETCH 1.0
static void delete_input(void);

// STATIC struct containing config info from Scheme layer
static FL_GLOBAL *In=NULL;

void FL_get_globals_from_lisp(LISP params){

  // Alloc new struct (free first if already exists)
  //  and assign to static In
  create_FL_GLOBAL_struct();
 
  // Get params from Scheme layer
  In->vibrato_freq = get_param_float("vibrato_freq",params,DEF_VIBRATO_FREQ);
  In->vibrato_max  = get_param_float("vibrato_max", params,DEF_VIBRATO_MAX);
  In->bend_semi_max  = get_param_float("bend_semi_max", params,DEF_BEND_SEMI_MAX);
  In->portamento_time_default  = get_param_float("portamento_time_default", params,DEF_PORTAMENTO_TIME_DEFAULT);
  In->drift_freq1 = get_param_float("drift_freq1",params,DEF_DRIFT_FREQ1);
  In->drift_freq2 = get_param_float("drift_freq2",params,DEF_DRIFT_FREQ2);
  In->drift_freq3 = get_param_float("drift_freq3",params,DEF_DRIFT_FREQ3);
  In->drift_ampl  = get_param_float("drift_ampl", params,DEF_DRIFT_AMPL); 
  In->transpose   = get_param_float("transpose", params,DEF_TRANSPOSE);
  In->consonant_stretch = get_param_float("consonant_stretch", params,DEF_CONSONANT_STRETCH);
}

void create_FL_GLOBAL_struct(void){

  FL_GLOBAL *new_in;

  // Alloc a new struct
  new_in = dballoc(FL_GLOBAL, 1);
  memset(new_in,0,sizeof(FL_GLOBAL));  // zero out

  // Defaults only
  //  -- these are only used if FL_get_globals_from_lisp is never called
  //     (i.e., if scheme layer never calls Flinger_Init)
  new_in->vibrato_freq  = DEF_VIBRATO_FREQ;
  new_in->vibrato_max   = DEF_VIBRATO_MAX;
  new_in->bend_semi_max = DEF_BEND_SEMI_MAX;
  new_in->portamento_time_default = DEF_PORTAMENTO_TIME_DEFAULT;
  new_in->drift_freq1 = DEF_DRIFT_FREQ1;
  new_in->drift_freq2 = DEF_DRIFT_FREQ2;
  new_in->drift_freq3 = DEF_DRIFT_FREQ3;
  new_in->drift_ampl  = DEF_DRIFT_AMPL;
  new_in->transpose   = DEF_TRANSPOSE;
  new_in->consonant_stretch = DEF_CONSONANT_STRETCH;

  if (In != NULL){
    delete_input();
  }

  // Assign to static   
  In = new_in;
}

static void delete_input(void){
  dbfree(In);
  In = NULL;
}

// Get ptr to struct
FL_GLOBAL *get_global_info(void){
    return(In);
}

int FL_GLOBAL_struct_exists(void){
  if (In != NULL)
    return 1;
  else
    return 0;
}

