/*-----------------------------------------------------------------------*/
/* Gets synthesis parameters from the lisp structs specified in Scheme   */
/*=======================================================================*/
#include <stdlib.h>
#include "festival.h"
#include "db_alloc.h"
#include "fl_io.h"

#define DEF_VIBRATO_FREQ 5.5
#define DEF_VIBRATO_MAX  0.05
#define DEF_BEND_SEMI_MAX 12.0
#define DEF_PORTAMENTO_TIME_DEFAULT 0.010
#define DEF_DRIFT_FREQ1 4.7
#define DEF_DRIFT_FREQ2 7.1
#define DEF_DRIFT_FREQ3 12.7
#define DEF_TRANSPOSE 0.0
#define DEF_DRIFT_AMPL 0.002
#define DEF_CONSONANT_STRETCH 1.0

// STATIC struct containing info from Scheme layer
static EST_Features *In=NULL;

void Flinger_init_params(LISP params){

  // Alloc new struct (free first if already exists)
  //  and assign to static In
  create_Flinger_input();
 
  // Get params from Scheme layer
  In->set("vibrato_freq",  get_param_float("vibrato_freq",params,DEF_VIBRATO_FREQ));
  In->set("vibrato_max",   get_param_float("vibrato_max", params,DEF_VIBRATO_MAX));
  In->set("bend_semi_max", get_param_float("bend_semi_max", params,DEF_BEND_SEMI_MAX));
  In->set("portamento_time_default", get_param_float("portamento_time_default", params,DEF_PORTAMENTO_TIME_DEFAULT));
  In->set("drift_freq1", get_param_float("drift_freq1",params,DEF_DRIFT_FREQ1));
  In->set("drift_freq2", get_param_float("drift_freq2",params,DEF_DRIFT_FREQ2));
  In->set("drift_freq3", get_param_float("drift_freq3",params,DEF_DRIFT_FREQ3));
  In->set("drift_ampl", get_param_float("drift_ampl", params,DEF_DRIFT_AMPL)); 
  In->set("transpose", get_param_float("transpose", params,DEF_TRANSPOSE));
  In->set("consonant_stretch", get_param_float("consonant_stretch", params,DEF_CONSONANT_STRETCH));
}


void Flinger_addto_params(LISP params){
  if (In == NULL)
    create_Flinger_input();
 
  LISP l,plisp;
  EST_String thing;
  for (l=params; l !=0; l=cdr(l)){
    thing = get_c_string(car(car(l)));
  
    // check to see if it's new
    if (!(In->present(thing))){ 
      In->set(thing, get_c_float(car(cdr(car((l))))));
      cout << "new input param \"" << thing << "\" added\n";
    }
    else { // else  must be float type
      In->set(thing, get_c_float(car(cdr(car((l))))));
    }
  }
}

EST_Features *get_Flinger_params(void){
  if (In == NULL)
    create_Flinger_input();

  return In;
}

void create_Flinger_input(void){
 
  EST_Features *new_in;

  new_in = new EST_Features;
  
  if (In != NULL){
    delete In;
  }
  
  // Assign to static   
  In = new_in;
}

// check to see if Flinger_init_params() has been called
bool Flinger_params_loaded(void){
  if (In == NULL)
    return(FALSE);
  else
    return(TRUE);
}
