// modified from original distrib   Print-->Parse
/*-
 * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Michael B. Durian.
 * 4. The name of the the Author may be used to endorse or promote 
 *    products derived from this software without specific prior written 
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
extern "C" {
#include <tcl.h>
}
/* Damn Microsoft compiler can't deal with the real name of the include file */
#ifndef _MSC_VER
#include <strstream.h>
#else
#include <strstrea.h>
#endif
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "MIDI/AllEvent.h"
#include "fl_midi_parse.h"
#include "fl_midi_ext.h"

void
Tclm_ParseEvent(Event *e) {
	switch (e->GetType()) {
	case NOTEOFF: //FL
		if (((NoteEvent *)e)->GetNotePair() != 0) {
		        return;
		}
		Tclm_ParseNoteOff((NoteOffEvent *)e);
		break;
	case NOTEON: //FL
		if (((NoteEvent *)e)->GetNotePair() == 0)
			Tclm_ParseNoteOn((NoteOnEvent *)e);
		else {
			if ((int)((NoteEvent *)e)->GetVelocity() == 0) {
				return;
			}
			Tclm_ParseNote((NoteOnEvent *)e);
		}
		break;
	case KEYPRESSURE: /// TO-DO - vocal effort
		Tclm_ParseKeyPressure((KeyPressureEvent *)e);
		break;
	case PARAMETER:  /// TO-DO - vibrato
		Tclm_ParseParameter((ParameterEvent *)e);
		break;
	case PROGRAM:
		Tclm_ParseProgram((ProgramEvent *)e);
		break;
	case CHANNELPRESSURE:
		Tclm_ParseChannelPressure((ChannelPressureEvent *)e);
		break;
	case PITCHWHEEL:/// TO-DO - pitch bend
		Tclm_ParsePitchWheel((PitchWheelEvent *)e);
		break;
	case SYSTEMEXCLUSIVE:
		Tclm_ParseSystemExclusive((SystemExclusiveEvent *)e);
		break;
	case METASEQUENCENUMBER:
		Tclm_ParseMetaSequenceNumber(
		    (MetaSequenceNumberEvent *)e);
		break;
	case METATEXT:
		Tclm_ParseMetaText((MetaTextEvent *)e);
		break;
	case METACOPYRIGHT:
		Tclm_ParseMetaCopyright((MetaCopyrightEvent *)e);
		break;
	case METASEQUENCENAME:
		Tclm_ParseMetaSequenceName((MetaSequenceNameEvent *)e);
		break;
	case METAINSTRUMENTNAME:
		Tclm_ParseMetaInstrumentName(
		    (MetaInstrumentNameEvent *)e);
		break;
	case METALYRIC: //FL
		Tclm_ParseMetaLyric((MetaLyricEvent *)e);
		break;
	case METAMARKER:
		Tclm_ParseMetaMarker((MetaMarkerEvent *)e);
		break;
	case METACUE:
		Tclm_ParseMetaCue((MetaCueEvent *)e);
		break;
	case METACHANNELPREFIX:
		Tclm_ParseMetaChannelPrefix((MetaChannelPrefixEvent *)e);
		break;
	case METAPORTNUMBER:
		Tclm_ParseMetaPortNumber((MetaPortNumberEvent *)e);
		break;
	case METAENDOFTRACK:
		Tclm_ParseMetaEndOfTrack((MetaEndOfTrackEvent *)e);
		break;
	case METATEMPO: //FL
		Tclm_ParseMetaTempo((MetaTempoEvent *)e);
		break;
	case METASMPTE:
		Tclm_ParseMetaSMPTE((MetaSMPTEEvent *)e);
		break;
	case METATIME:
		Tclm_ParseMetaTime((MetaTimeEvent *)e);
		break;
	case METAKEY: //FL
		Tclm_ParseMetaKey((MetaKeyEvent *)e);
		break;
	case METASEQUENCERSPECIFIC:
		Tclm_ParseMetaSequencerSpecific(
		    (MetaSequencerSpecificEvent *)e);
		break;
	case METAUNKNOWN:
		Tclm_ParseMetaUnknown((MetaUnknownEvent *)e);
		break;
	default:
	  cerr << "fl_midi_parse: unrecognized MIDI event\n";
	  break;
	}
}

void
Tclm_ParseNoteOff(NoteOffEvent *e)
{
	
	int c, p, v;

	c = e->GetChannel();
	p = e->GetPitch();
	v = e->GetVelocity();
	cerr << "NoteOff without NoteOn?";
	
	return;
}

void
Tclm_ParseNoteOn(NoteOnEvent *e)
{
	
	int c, p, v;

	c = e->GetChannel();
	p = e->GetPitch();
	v = e->GetVelocity();
	cerr << "NoteOn without NoteOff?";
	//cerr << "NoteOn ";
	
	return;
}

void
Tclm_ParseNote(NoteOnEvent *e)
{
	//cerr << "Note ";	
	
	// add a note to the Utterance
	FL_add_note(
		    e->GetTime(),
		    e->GetChannel(),
		    e->GetPitch(),  
		    e->GetDuration(),
		    e->GetVelocity()
		    );
	return;
}

void
Tclm_ParseMetaLyric(MetaLyricEvent *e)
{
	//cerr << "MetaLyric ";
	FL_add_lyric_token(e->GetTime(),e->GetString());
	
	return;
}

void
Tclm_ParseParameter(ParameterEvent *e)
{
	int p = e->GetParameter();
	
	switch (p){
	case 1:   // Modulation 
	  FL_add_vibrato( e->GetTime(),
			  e->GetChannel(),
			  e->GetValue());
	  break;
	case 7:   // Volume
	  //	  cerr << " vocal effort signal\n";
	  break;
	}
	
	//cerr << "Parameter ";
	
	return;
}


void 
Tclm_ParseMetaTempo(MetaTempoEvent *e){
  //cerr << "MetaTempo ";
  FL_update_tempo(e->GetTime(), e->GetTempo());
  return;
}


void
Tclm_ParsePitchWheel(PitchWheelEvent *e)
{
	//cerr << "PitchWheel ";
	FL_add_pitchbend(
		    e->GetTime(),
		    e->GetChannel(),
		    e->GetValue()
		    );
	return;
}



///////////// below this not used yet ///////////////

void
Tclm_ParseKeyPressure(KeyPressureEvent *e)
{
	
	int c, p;

	c = e->GetChannel();
	p = e->GetPitch();
	//cerr << "KeyPressure ";
	
	return;
}

void
Tclm_ParseProgram(ProgramEvent *e)
{
	
	int c, v;

	c = e->GetChannel();
	v = e->GetValue();
	//cerr << "Program ";
	
	return;
}

void
Tclm_ParseChannelPressure(ChannelPressureEvent *e)
{
	
	int c, p;

	c = e->GetChannel();
	p = e->GetPressure();

	//cerr << "ChannelPressure ";
	
	return;
}

void
Tclm_ParseSystemExclusive(SystemExclusiveEvent *e)
{
	
	const unsigned char *d;
	long l;

	d = e->GetData();
	l = e->GetLength();

	//cerr << "SystemExclusive ";
	
	return;
}

void
Tclm_ParseMetaSequenceNumber(MetaSequenceNumberEvent *e)
{
	
	long l;

	l = e->GetNumber();

	//cerr << "MetaSequenceNumber ";
	
	return;
}

void
Tclm_ParseMetaText(MetaTextEvent *e)
{
	
	const char *s;

	s = e->GetString();

	//cerr << "MetaText ";
	
	return;
}

void
Tclm_ParseMetaCopyright(MetaCopyrightEvent *e)
{
	
	const char *s;

	s = e->GetString();
	//cerr << "MetaCopyright ";
	
	return;
}

void
Tclm_ParseMetaSequenceName(MetaSequenceNameEvent *e)
{
	
	const char *s;

	s = e->GetString();
	//cerr << "MetaSequenceName ";
	
	return;
}

void
Tclm_ParseMetaInstrumentName(MetaInstrumentNameEvent *e)
{
	
	const char *s;

	s = e->GetString();
	//cerr << "MetaInstrumentName ";
	
	return;
}

void
Tclm_ParseMetaMarker(MetaMarkerEvent *e)
{
	
	const char *s;

	s = e->GetString();
	//cerr << "MetaMarker ";
	
	return;
}

void
Tclm_ParseMetaCue(MetaCueEvent *e)
{
	
	const char *s;

	s = e->GetString();
	//cerr << "MetaCue ";
	
	return;
}

void
Tclm_ParseMetaChannelPrefix(MetaChannelPrefixEvent *e)
{
	
	const unsigned char *d;

	d = e->GetData();
	//cerr << "MetaChannelPrefix ";
	
	return;
}

void
Tclm_ParseMetaPortNumber(MetaPortNumberEvent *e)
{
	
	int p;

	p = e->GetPort();
	//cerr << "MetaPortNumber ";
	
	return;
}

void
Tclm_ParseMetaEndOfTrack(MetaEndOfTrackEvent *e)
{
	
	MetaEndOfTrackEvent *dummy;

	// shut up a warning
	dummy = e;

	//cerr << "MetaEndOfTrack" << ends;
	return;
}

void
Tclm_ParseMetaSMPTE(MetaSMPTEEvent *e)
{
	
	int h, m, s, f, ff;

	h = e->GetHour();
	m = e->GetMinute();
	s = e->GetSecond();
	f = e->GetFrame();
	ff = e->GetFractionalFrame();
	//cerr << "MetaSMPTE ";
	
	return;
}

void
Tclm_ParseMetaTime(MetaTimeEvent *e)
{
	
	int n, d, c, t;

	n = e->GetNumerator();
	d = e->GetDenominator();
	c = e->GetClocksPerBeat();
	t = e->Get32ndNotesPerQuarterNote();

	//cerr << "MetaTime ";
	
	return;
}

void
Tclm_ParseMetaKey(MetaKeyEvent *e)
{
  //	cerr << "MetaKey ";
  //	cerr << "{" << e->GetKeyStr() << "}";
	//cerr << e->GetModeStr();
	return;
}

void
Tclm_ParseMetaSequencerSpecific(MetaSequencerSpecificEvent *e)
{
	
	const unsigned char *d;

	d = e->GetData();

	//cerr << "MetaSequencerSpecific ";
	
	return;
}

void
Tclm_ParseMetaUnknown(MetaUnknownEvent *e)
{
	
	int t;
	const unsigned char *d;

	t = e->GetMetaType();
	d = e->GetData();

	//cerr << "MetaUnknown ";
	
	return;
}

//// from tclmutil.cxx  MWM

void
Tclm_ParseData(const unsigned char *data, long length)
{
        long i;
	for (i = 1; i < length; i++)
	  cerr << " " << (int)data[i];
	
	//buf.setf(ios::showbase | ios::internal);
        //buf << hex << setw(4) << setfill('0') << (int)data[0];
        //for (i = 1; i < length; i++)
	//       buf << " " << hex << setw(4) << setfill('0') << (int)data[i];
}
