/*-
 * Copyright (c) 1993, 1994, 1995, 1996 Michael B. Durian.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Michael B. Durian.
 * 4. The name of the the Author may be used to endorse or promote 
 *    products derived from this software without specific prior written 
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
extern "C" {
#include <tcl.h>
}
/* Damn Microsoft compiler can't deal with the real name of the include file */
#ifndef _MSC_VER
#include <strstream.h>
#else
#include <strstrea.h>
#endif
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "MIDI/AllEvent.h"
#include "fl_midi_print.h"

void
Tclm_PrintEvent(ostream &buf, Event *e)
{
	char *str;

	switch (e->GetType()) {
	case NOTEOFF:
		if (((NoteEvent *)e)->GetNotePair() != 0) {
			buf << ends;
			return;
		}
		str = Tclm_PrintNoteOff((NoteOffEvent *)e);
		break;
	case NOTEON:
		if (((NoteEvent *)e)->GetNotePair() == 0)
			str = Tclm_PrintNoteOn((NoteOnEvent *)e);
		else {
			if ((int)((NoteEvent *)e)->GetVelocity() == 0) {
				buf << ends;
				return;
			}
			str = Tclm_PrintNote((NoteOnEvent *)e);
		}
		break;
	case KEYPRESSURE:
		str = Tclm_PrintKeyPressure((KeyPressureEvent *)e);
		break;
	case PARAMETER:
		str = Tclm_PrintParameter((ParameterEvent *)e);
		break;
	case PROGRAM:
		str = Tclm_PrintProgram((ProgramEvent *)e);
		break;
	case CHANNELPRESSURE:
		str = Tclm_PrintChannelPressure((ChannelPressureEvent *)e);
		break;
	case PITCHWHEEL:
		str = Tclm_PrintPitchWheel((PitchWheelEvent *)e);
		break;
	case SYSTEMEXCLUSIVE:
		str = Tclm_PrintSystemExclusive((SystemExclusiveEvent *)e);
		break;
	case METASEQUENCENUMBER:
		str = Tclm_PrintMetaSequenceNumber(
		    (MetaSequenceNumberEvent *)e);
		break;
	case METATEXT:
		str = Tclm_PrintMetaText((MetaTextEvent *)e);
		break;
	case METACOPYRIGHT:
		str = Tclm_PrintMetaCopyright((MetaCopyrightEvent *)e);
		break;
	case METASEQUENCENAME:
		str = Tclm_PrintMetaSequenceName((MetaSequenceNameEvent *)e);
		break;
	case METAINSTRUMENTNAME:
		str = Tclm_PrintMetaInstrumentName(
		    (MetaInstrumentNameEvent *)e);
		break;
	case METALYRIC:
		str = Tclm_PrintMetaLyric((MetaLyricEvent *)e);
		break;
	case METAMARKER:
		str = Tclm_PrintMetaMarker((MetaMarkerEvent *)e);
		break;
	case METACUE:
		str = Tclm_PrintMetaCue((MetaCueEvent *)e);
		break;
	case METACHANNELPREFIX:
		str = Tclm_PrintMetaChannelPrefix((MetaChannelPrefixEvent *)e);
		break;
	case METAPORTNUMBER:
		str = Tclm_PrintMetaPortNumber((MetaPortNumberEvent *)e);
		break;
	case METAENDOFTRACK:
		str = Tclm_PrintMetaEndOfTrack((MetaEndOfTrackEvent *)e);
		break;
	case METATEMPO:
		str = Tclm_PrintMetaTempo((MetaTempoEvent *)e);
		break;
	case METASMPTE:
		str = Tclm_PrintMetaSMPTE((MetaSMPTEEvent *)e);
		break;
	case METATIME:
		str = Tclm_PrintMetaTime((MetaTimeEvent *)e);
		break;
	case METAKEY:
		str = Tclm_PrintMetaKey((MetaKeyEvent *)e);
		break;
	case METASEQUENCERSPECIFIC:
		str = Tclm_PrintMetaSequencerSpecific(
		    (MetaSequencerSpecificEvent *)e);
		break;
	case METAUNKNOWN:
		str = Tclm_PrintMetaUnknown((MetaUnknownEvent *)e);
		break;
	default:
		str = 0;
		break;
	}
	if (e->GetTime() == Event::WC_TIME)
		buf << "* " << str << ends;
	else
		buf << e->GetTime() << " " << str << ends;
	delete str;
}

char *
Tclm_PrintNoteOff(NoteOffEvent *e)
{
	ostrstream buf;
	int c, p, v;

	c = e->GetChannel();
	p = e->GetPitch();
	v = e->GetVelocity();
	buf << "NoteOff ";
	if (c == NoteOffEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (p == NoteOffEvent::WC_PITCH)
		buf << "*";
	else
		buf << p;
	buf << " ";
	if (v == NoteOffEvent::WC_VELOCITY)
		buf << "*";
	else
		buf << v;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintNoteOn(NoteOnEvent *e)
{
	ostrstream buf;
	int c, p, v;

	c = e->GetChannel();
	p = e->GetPitch();
	v = e->GetVelocity();
	buf << "NoteOn ";
	if (c == NoteOnEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (p == NoteOnEvent::WC_PITCH)
		buf << "*";
	else
		buf << p;
	buf << " ";
	if (v == NoteOnEvent::WC_VELOCITY)
		buf << "*";
	else
		buf << v;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintNote(NoteOnEvent *e)
{
	ostrstream buf;
	int c, p, v;
	unsigned long d;

	c = e->GetChannel();
	p = e->GetPitch();
	v = e->GetVelocity();
	d = e->GetDuration();
	buf << "Note ";
	if (c == NoteOnEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (p == NoteOnEvent::WC_PITCH)
		buf << "*";
	else
		buf << p;
	buf << " ";
	if (v == NoteOnEvent::WC_VELOCITY)
		buf << "*";
	else
		buf << v;
	buf << " ";
	if (d == NoteOnEvent::WC_DURATION)
		buf << "*";
	else
		buf << d;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintKeyPressure(KeyPressureEvent *e)
{
	ostrstream buf;
	int c, p;

	c = e->GetChannel();
	p = e->GetPitch();
	buf << "KeyPressure ";
	if (c == KeyPressureEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (p == KeyPressureEvent::WC_PITCH)
		buf << "*";
	else
		buf << p;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintParameter(ParameterEvent *e)
{
	ostrstream buf;
	int c, p, v;

	c = e->GetChannel();
	p = e->GetParameter();
	v = e->GetValue();

	buf << "Parameter ";
	if (c == ParameterEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (p == ParameterEvent::WC_PARAMETER)
		buf << "*";
	else
		buf << p;
	buf << " ";
	if (v == ParameterEvent::WC_VALUE)
		buf << "*";
	else
		buf << v;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintProgram(ProgramEvent *e)
{
	ostrstream buf;
	int c, v;

	c = e->GetChannel();
	v = e->GetValue();
	buf << "Program ";
	if (c == ProgramEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (v == ProgramEvent::WC_VALUE)
		buf << "*";
	else
		buf << v;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintChannelPressure(ChannelPressureEvent *e)
{
	ostrstream buf;
	int c, p;

	c = e->GetChannel();
	p = e->GetPressure();

	buf << "ChannelPressure ";
	if (c == ChannelPressureEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (p == ChannelPressureEvent::WC_PRESSURE)
		buf << "*";
	else
		buf << p;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintPitchWheel(PitchWheelEvent *e)
{
	ostrstream buf;
	int c;
	long v;

	c = e->GetChannel();
	v = e->GetValue();

	buf << "PitchWheel ";
	if (c == PitchWheelEvent::WC_CHANNEL)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (v == PitchWheelEvent::WC_VALUE)
		buf << "*";
	else
		buf << v;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintSystemExclusive(SystemExclusiveEvent *e)
{
	ostrstream buf;
	const unsigned char *d;
	long l;

	d = e->GetData();
	l = e->GetLength();

	buf << "SystemExclusive ";
	if (e->GetContinued() == 1)
		buf << "continued ";
	if (d == SystemExclusiveEvent::WC_DATA)
		buf << "*";
	else {
		buf << "{";
		Tclm_PrintData(buf, e->GetData(), e->GetLength());
		buf << "}";
	}
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaSequenceNumber(MetaSequenceNumberEvent *e)
{
	ostrstream buf;
	long l;

	l = e->GetNumber();

	buf << "MetaSequenceNumber ";
	if (l == MetaSequenceNumberEvent::WC_NUMBER)
		buf << "*";
	else
		buf << l;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaText(MetaTextEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();

	buf << "MetaText ";
	if (s == MetaTextEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaCopyright(MetaCopyrightEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();
	buf << "MetaCopyright ";
	if (s == MetaCopyrightEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaSequenceName(MetaSequenceNameEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();
	buf << "MetaSequenceName ";
	if (s == MetaSequenceNameEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaInstrumentName(MetaInstrumentNameEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();
	buf << "MetaInstrumentName ";
	if (s == MetaInstrumentNameEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaLyric(MetaLyricEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();
	buf << "MetaLyric ";
	if (s == MetaLyricEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaMarker(MetaMarkerEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();
	buf << "MetaMarker ";
	if (s == MetaMarkerEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaCue(MetaCueEvent *e)
{
	ostrstream buf;
	const char *s;

	s = e->GetString();
	buf << "MetaCue ";
	if (s == MetaCueEvent::WC_STRING)
		buf << "*";
	else
		buf << "{" << s << "}";
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaChannelPrefix(MetaChannelPrefixEvent *e)
{
	ostrstream buf;
	const unsigned char *d;

	d = e->GetData();
	buf << "MetaChannelPrefix ";
	if (d == MetaChannelPrefixEvent::WC_DATA)
		buf << "*";
	else {
		buf << "{";
		Tclm_PrintData(buf, d, e->GetLength());
		buf << "}";
	}
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaPortNumber(MetaPortNumberEvent *e)
{
	ostrstream buf;
	int p;

	p = e->GetPort();
	buf << "MetaPortNumber ";
	if (p == MetaPortNumberEvent::WC_PORT)
		buf << "*";
	else
		buf << p;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaEndOfTrack(MetaEndOfTrackEvent *e)
{
	ostrstream buf;
	MetaEndOfTrackEvent *dummy;

	// shut up a warning
	dummy = e;

	buf << "MetaEndOfTrack" << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaTempo(MetaTempoEvent *e)
{
	ostrstream buf;
	short t;

	t = e->GetTempo();
	buf << "MetaTempo ";
	if (t == MetaTempoEvent::WC_TEMPO)
		buf << "*";
	else
		buf << t;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaSMPTE(MetaSMPTEEvent *e)
{
	ostrstream buf;
	int h, m, s, f, ff;

	h = e->GetHour();
	m = e->GetMinute();
	s = e->GetSecond();
	f = e->GetFrame();
	ff = e->GetFractionalFrame();
	buf << "MetaSMPTE ";
	if (h == MetaSMPTEEvent::WC_HOUR)
		buf << "*";
	else
		buf << h;
	buf << " ";
	if (m == MetaSMPTEEvent::WC_MINUTE)
		buf << "*";
	else
		buf << m;
	buf << " ";
	if (s == MetaSMPTEEvent::WC_SECOND)
		buf << "*";
	else
		buf << s;
	buf << " ";
	if (f == MetaSMPTEEvent::WC_FRAME)
		buf << "*";
	else
		buf << f;
	buf << " ";
	if (ff == MetaSMPTEEvent::WC_FRACTIONAL_FRAME)
		buf << "*";
	else
		buf << ff;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaTime(MetaTimeEvent *e)
{
	ostrstream buf;
	int n, d, c, t;

	n = e->GetNumerator();
	d = e->GetDenominator();
	c = e->GetClocksPerBeat();
	t = e->Get32ndNotesPerQuarterNote();

	buf << "MetaTime ";
	if (n == MetaTimeEvent::WC_NUMERATOR)
		buf << "*";
	else
		buf << n;
	buf << " ";
	if (d == MetaTimeEvent::WC_DENOMINATOR)
		buf << "*";
	else
		buf << d;
	buf << " ";
	if (c == MetaTimeEvent::WC_CLOCKS_PER_BEAT)
		buf << "*";
	else
		buf << c;
	buf << " ";
	if (t == MetaTimeEvent::WC_32ND_NOTES_PER_QUARTER_NOTE)
		buf << "*";
	else
		buf << t;
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaKey(MetaKeyEvent *e)
{
	ostrstream buf;

	buf << "MetaKey ";
	if (e->GetKey() == MetaKeyEvent::WC_KEY)
		buf << "*";
	else
		buf << "{" << e->GetKeyStr() << "}";
	buf << " ";
	if (e->GetMode() == MetaKeyEvent::WC_MODE)
		buf << "*";
	else
		buf << e->GetModeStr();
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaSequencerSpecific(MetaSequencerSpecificEvent *e)
{
	ostrstream buf;
	const unsigned char *d;

	d = e->GetData();

	buf << "MetaSequencerSpecific ";
	if (d == MetaSequencerSpecificEvent::WC_DATA)
		buf << "*";
	else {
		buf << "{";
		Tclm_PrintData(buf, d, e->GetLength());
		buf << "}";
	}
	buf << ends;
	return (buf.str());
}

char *
Tclm_PrintMetaUnknown(MetaUnknownEvent *e)
{
	ostrstream buf;
	int t;
	const unsigned char *d;

	t = e->GetMetaType();
	d = e->GetData();

	buf << "MetaUnknown ";
	if (t == MetaUnknownEvent::WC_META_TYPE)
		buf << "*";
	else
		buf << t;
	buf << " ";
	if (d == MetaUnknownEvent::WC_DATA)
		buf << "*";
	else {
		buf << "{";
		Tclm_PrintData(buf, d, e->GetLength());
		buf << "}";
	}
	buf << ends;
	return (buf.str());
}

//// from tclmutil.cxx  MWM

void
Tclm_PrintData(ostream &buf, const unsigned char *data, long length)
{
        long i;
 
        buf.setf(ios::showbase | ios::internal);
        buf << hex << setw(4) << setfill('0') << (int)data[0];
        for (i = 1; i < length; i++)
                buf << " " << hex << setw(4) << setfill('0') << (int)data[i];
}
