#include <stdlib.h>
#include "festival.h"
#include "fl_io.h"
#include "fl_pitch.h"
#include "OGI_Macro.h"

static void relate_F0_Mod_to_Note(EST_Utterance *u);
static void add_note_target(EST_Utterance *u, EST_Item *note, F0mod_obj &mod);

static float F0_SampPeriod;   // sampling rate for F0 contour
static float DefPortaTime;  // default portamento time

LISP FL_NoteF0_Utt(LISP utt){
  cout << "* FL_NoteF0\n";
  
  EST_Item *note;
  EST_Utterance *u = get_c_utt(utt);
  EST_Features *gl = get_Flinger_params();

  F0mod_obj mod;

  // set globals
  F0_SampPeriod = mod.nyquist_period();
  DefPortaTime = gl->F("portamento_time_default");

  // relate events in the F0_Mod relation to the Note relation
  relate_F0_Mod_to_Note(u);

  // loop through notes, set F0 Targets at begin and end of each for synthesis
  for (note=u->relation("Note")->first(); note != 0; note = next(note)){
    add_note_target(u, note, mod);
  }

  return utt;
}

static void relate_F0_Mod_to_Note(EST_Utterance *u){
  
  EST_Item *f0mod,*n;
  float next_ntime,t_vibr;
  
  f0mod=u->relation("F0_Mod")->first();
  for (n=u->relation("Note")->first(); n != 0; n=next(n)) {

    next_ntime = n->F("off");

    if (f0mod != NULL){
      t_vibr = f0mod->F("start");
      while (t_vibr < next_ntime){
	append_daughter(n, "NoteStruct", f0mod);
	if ((f0mod = next(f0mod)) == NULL)
	  break;
	t_vibr = f0mod->F("start");
      }
    }
  }
}


/*

  basic idea:  
  - make/get relations of PitchBend and Vibrato items to Notes
  - for each note
      - take note of time of next update of PB and/or V
      - step along at F0 sampling rate until t_end-glidetime
      - for each step, ask for F0_mod(t,Fo_in)  <== make an object
      - when next event passed, update F0 mod params (in obj)
      

*/


#define EPSILON 0.000001
static void add_note_target(EST_Utterance *u, EST_Item *note, F0mod_obj &modOsc){

  float samp_period = F0_SampPeriod; 
  EST_Item *Fo_targ;
  float Fo,t_on,t_off,t_end,Fo_out;
  float t,t_next_event;
  note = note->as_relation("NoteStruct");
  EST_Relation *TargRel=NULL;

  // get vibrato and F0_Mod events related to this note
  EST_Item *f0mod = daughter1(note);
  
  Fo    = note->F("freq");
  t_on  = note->F("on");
  t_off = note->F("off");
  //  cout << ": " << t_on << ", " << t_off << "\t" << Fo << "*" << modOsc.get(t_on) << endl;
  t_end = max(t_off-DefPortaTime, t_on+EPSILON);
  
  t=t_on; 
  while (t < t_end){
    //
    // Get the next F0 modulation event
    //
    t_next_event = t_off;  
    if (f0mod != NULL){
      t_next_event = f0mod->F("start");  
    }
    f0mod = next(f0mod);

    //
    //  Add (F0) Target events at sampling rate
    //
    TargRel = u->relation("Target");
    while ((t < t_next_event) && (t < t_end)){
      Fo_targ = TargRel->append();
      Fo_out = Fo*modOsc.get(t);
      Fo_targ->set("f0",Fo_out);
      Fo_targ->set("pos",t);
      t+=samp_period;
    }
    
    //
    // Update parameters of the F0 modulation model
    //
    if ((t < t_end) && (f0mod != NULL)){
      if (f0mod->name() == "pitchbend")
	modOsc.update_bend(f0mod->F("percent"));
      else if (f0mod->name() == "vibrato")
	modOsc.update_vA(f0mod->F("percent"));
      else {
	cerr << "unknown type of F0_Mod event :  " << f0mod->name() << endl; 
      }
    }
    
  }
  

  // make sure there is one at time t_end
  if ((t-samp_period < t_end) && (TargRel)){    
    Fo_targ = TargRel->append();
    Fo_out = Fo*modOsc.get(t_end);     
    Fo_targ->set("f0",Fo_out);
    Fo_targ->set("pos",t_end);
  }
  
  // make sure we have updated last of F0_Mod events
  f0mod = f0mod->last();
  if (f0mod != NULL){
    if (f0mod->name() == "pitchbend")
      modOsc.update_bend(f0mod->F("percent"));
    else if (f0mod->name() == "vibrato")
      modOsc.update_vA(f0mod->F("percent"));
    else {
      cerr << "unknown type of F0_Mod event :  " << f0mod->name() << endl; 
    }
  }
}

