
#ifndef __FL_PITCH_H__
#define __FL_PITCH_H__

#include "db_alloc.h"
#include "fl_io.h" 
#include "OGI_Macro.h" 

LISP FL_NoteF0_Utt(LISP utt);

#define TWOPI 6.2831853

class F0mod_obj {
  //
  // parameters labeled "current" are dynamically 
  //  controllable from MIDI
  //
private:
  
  float dF1;  // drift freq 1 (Hz)
  float dF2;  // drift freq 2 (Hz)
  float dF3;  // drift freq 3 (Hz)
  float dA;   // drift amplitude (%Fo)

  float vF;  // vibrato freq
  float vA;  // current vibrato ampl (%Fo)

  float bend;  // current pitchbend value (%Fo)
  float transpose; // transpose shift (%Fo)

public:
  F0mod_obj(void){
    EST_Features *gl = get_Flinger_params();
    dF1 = gl->F("drift_freq1");
    dF2 = gl->F("drift_freq2");
    dF3 = gl->F("drift_freq3");
    dA  = gl->F("drift_ampl");
    vF  = gl->F("vibrato_freq");
    transpose = (float) pow(2.0,gl->F("transpose")/12.0);

    vA = 0.0;
    bend = 1.0;  
  }

  float get(float t){
    // returns a factor to multiply by Fo
    return transpose*bend*(1 + dA*drift(t) + vA*vibrato(t));
  }

  float nyquist_period(void){ 
    // returns necessary no-alias sampling period in seconds
    return 1.0/(2.0*max(max(max(dF1,dF2),dF3),vF));
  }

  float drift(float t){
    return cos(TWOPI*dF1*t)+cos(TWOPI*dF2*t)+cos(TWOPI*dF3*t);
  }

  float vibrato(float t){
    return cos(TWOPI*vF*t);
  }

  void update_vA(float vA_in){
    vA = vA_in;
  }
  
  void update_bend(float bend_in){
    bend = bend_in;
  }
  
};
    
#endif
