
//
//  Some functions in this file adapted from 
//                   festival/src/modules/base/word.cc
//                   festival/modules/Text/token.cc
//

/*************************************************************************/
/*                                                                       */
/*                Centre for Speech Technology Research                  */
/*                     University of Edinburgh, UK                       */
/*                       Copyright (c) 1996,1997                         */
/*                        All Rights Reserved.                           */
/*                                                                       */
/*  Permission is hereby granted, free of charge, to use and distribute  */
/*  this software and its documentation without restriction, including   */
/*  without limitation the rights to use, copy, modify, merge, publish,  */
/*  distribute, sublicense, and/or sell copies of this work, and to      */
/*  permit persons to whom this work is furnished to do so, subject to   */
/*  the following conditions:                                            */
/*   1. The code must retain the above copyright notice, this list of    */
/*      conditions and the following disclaimer.                         */
/*   2. Any modifications must be clearly marked as such.                */
/*   3. Original authors' names are not deleted.                         */
/*   4. The authors' names are not used to endorse or promote products   */
/*      derived from this software without specific prior written        */
/*      permission.                                                      */
/*                                                                       */
/*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
/*  SHALL THE UNIVERSITY OF EDINBURGH NOR THE CONTRIBUTORS BE LIABLE     */
/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
/*  THIS SOFTWARE.                                                       */
/*                                                                       */
/*************************************************************************/

//  Strips off (and keeps track of) punctuation symbols, etc.
//  in LyricTokens.  Results in Lyric relation containing
//  pronouncable words/syllables, no punctuation or symbols.
#include <stdlib.h>
#include "festival.h"
#include "modules.h"
#include "lexicon.h"
static void relate_notes_to_lyrictokens(EST_Utterance *u);

static LISP lyric_it(EST_Item *t,const EST_String tok);
static LISP builtin_lyric_it(EST_Item *token, EST_String tok);

static LISP user_lyrictoken_to_lyric_func = NIL;
static EST_Regex RXpunctuation("\\(\\]\\|[-[.,!?]\\)+");
static EST_Regex RXnotpunctuation("[^.,-]");  // keep simple for now
static EST_String remove_punct(const EST_String &tok);
static EST_String remove_lead_trail_whitespace(const EST_String &tok);
static EST_String return_post_punct(const EST_String &tok);
static EST_String return_pre_punct(const EST_String &tok);
static EST_Item *add_lyric(EST_Utterance *u, const EST_String &name);
static EST_Item *add_lyric(EST_Utterance *u, LISP lyric);

LISP FL_Tokenize_Utt(LISP utt){
  cout << "* FL_Tokenize\n";
  
  // Uses user-specified token to lyric function (defined in scheme) 
  // or a set of builtin rules.   User specified one can also call
  // the built-in set.
  EST_Utterance *u = get_c_utt(utt);
  LISP lyrics,w;
  EST_Item *t;
  EST_Item *new_lyric;

  // relate notes to lyric tokens
  relate_notes_to_lyrictokens(u);

  // pull lyrictoken_to_lyric function definition out of scm environment
  user_lyrictoken_to_lyric_func = siod_get_lval("lyrictoken_to_lyric_func",NULL);
 
  for (t=u->relation("LyricToken")->first(); t != 0; t = next(t)){

    // record the punctuation associated with each token
    //  ..not sure I need this.
    //    t->set("pre-punc",return_pre_punct(t->name()));
    //    t->set("post-punc",return_post_punct(t->name()));

    // if detect "-", then do fancy re-assemble, lex lookup, dis-assemble
    //   if this is successful, make result look like /.../ "phonemes" inputs
    //   on LyricTokens.

    // convert lyrictoken name to 1 or more lyrics
    lyrics = lyric_it(t,t->name());
    
    // create item for each lyric, make daughter to LyricToken
    //   in both NoteStruct and SylStructure
    for (w=lyrics; w != NIL; w=cdr(w)){
      new_lyric = add_lyric(u,car(w));
      append_daughter(t,"SylStructure", new_lyric);
      append_daughter(t,"NoteStruct",  new_lyric);
    }
  }
  user_lyrictoken_to_lyric_func = NIL;  // reset this

  return utt;
}

//
// Note->LyricToken association to get timing
//
static void relate_notes_to_lyrictokens(EST_Utterance *u){
  EST_Item *l,*note;
  float next_lyr_time, note_on;
  
  note=u->relation("Note")->first();
  for (l=u->relation("LyricToken")->first(); l != 0; l=next(l)) {

    u->relation("NoteWordAssoc")->append(l);

    if (next(l) != NULL)
      next_lyr_time = next(l)->F("time");
    else
      next_lyr_time = u->relation("Note")->last()->F("off");
    
    note_on = note->F("on");
    while (note_on < next_lyr_time){
      append_daughter(l,"NoteWordAssoc", note);
      if ((note = next(note)) == NULL)
	break;
      note_on = note->F("on");
    }
  }
}

 
LISP l_lyric_it(LISP token, LISP tok){
    // Lisp wrap around for lyric_it
    EST_Item *t = get_c_item(token);
    EST_String tok_name = get_c_string(tok);

    return builtin_lyric_it(t,tok_name);
}

static LISP lyric_it(EST_Item *token, const EST_String tok){
    // The user may specify his/her own additional token to lyric rules
    // through the variable user_lyrictoken_to_lyric_func. If so we must
    // call that, which may or may not call the builtin version
    // The builtin version is bound in LISP so that recursion works
    // properly
    // This takes a LISP utt as an argument as creating a new wraparound
    // will cause gc to fail.
    LISP tok_string = strcons(tok.length(),tok);

    if (user_lyrictoken_to_lyric_func != NIL)            // check user's rules
	return leval(cons(user_lyrictoken_to_lyric_func,
			  cons(siod(token),
			       cons(tok_string,NIL))),NIL);
    else
	return builtin_lyric_it(token,tok);
}
static LISP builtin_lyric_it(EST_Item *token, EST_String tok){
  // Return a list of lyrics for this token 
  remove_lead_trail_whitespace(tok);

  if (tok == "")
    return NIL;

  else if (tok.contains("/")) {
    /////////////////////////////////////////////////
    //////  used to give phoneme input  /////////////
    /////////////////////////////////////////////////
    EST_String slist,s;
    EST_String t;

    tok.gsub(RXwhite, "");  // rm all whitespaces

    //   strip off the /'s and create the 
    //    text equivalent of a LISP list
    //    cout << "*" << tok << "*\n";
    slist = "(";
    while (tok != ""){
      if (tok.contains("/")) 
	t=tok.before("/");
      else
	t = tok;
      slist += t.quote((const char)34);  // double quote
      slist += " ";
      tok = tok.after("/");
    }
    slist += ")";
    //    cout << slist << endl;

    // set the "phonemes" feature to be the list
    token->set("phonemes",slist);
    
    return cons(strintern("NONE"),NIL);
  }

  ///////////////////////////////////////////////////////
  else if (tok.contains("-")) {
    //  still existing hyphenation inside a lyric:
    //    strip off the "-" and separate into 2 lyric items
    //    BUT. still not sure what we will do with that later...

    cout << "   still found '-' \n";
    if (tok.before("-") == ""){
      if (tok.after("-") == "")
	return NIL;
      else 
	return lyric_it(token, tok.after("-"));
    }
    else {

      cout << "     found internal '-' ... improve this part later\n";

      if (tok.after("-") == "")
	return lyric_it(token, tok.before("-"));
      else 
	return append(lyric_it(token, tok.before("-")),
			lyric_it(token, tok.after("-")));
    }
  }
  
  ///////////////////////////////////////////////////////
  else if (tok.contains(RXwhite)) {
    //  whitespace inside a lyric:
    // separate into 2 lyric items
    
    return append(lyric_it(token, tok.before(RXwhite)),
		    lyric_it(token, tok.after(RXwhite)));
  }

  ///////////////////////////////////////////////////////  
  else{   
    // last resort: just take the lyric, drop all other punct
    return cons(strintern(downcase(remove_punct(tok))),NIL);
  }
}

static EST_String remove_punct(const EST_String &tok)
{
    EST_String np(tok);
    np.make_updatable();
    np.gsub(RXpunctuation, "");
    return np;
}
static EST_String remove_lead_trail_whitespace(const EST_String &tok){
  // removes leading and trailing whitespace
    EST_String np(tok);
    np.make_updatable();
    if (np.contains(RXwhite,0))
      np = np.after(RXwhite);
    if (np.contains(RXwhite,np.length()))
      np = np.before(RXwhite);
    return np;
}

///static EST_String return_punct(const EST_String &tok){
///  EST_String punc(tok);
///  punc.make_updatable();
///
///  // return only the beginning punctuation
///  punc.gsub(RXnotpunctuation, "");
///
///  return punc;
///}

static EST_String return_pre_punct(const EST_String &tok){
  EST_String punc = tok.before(RXnotpunctuation);
  return punc;
}

static EST_String return_post_punct(const EST_String &tok){
  

  cout << "return_post_punct:  this is WRONG!\n";
  EST_String punc = tok.after(RXnotpunctuation);
  return punc;
}


// the following 2 functions are based on festival/modules/base/word.cc

EST_Item *add_lyric(EST_Utterance *u, const EST_String &name){
  EST_Item *item = u->relation("Word")->append();
  item->set_name(name);    
  return item;
}

EST_Item *add_lyric(EST_Utterance *u, LISP lyric)
{
    // Build a Lyric item from the Lisp description, which may
    // contain other features
    LISP f;
    EST_Item *si_lyric;
    int has_name = FALSE;

    if (consp(lyric))
    {
	// feature form
	si_lyric = add_lyric(u,"");
	for (f=lyric; f != NIL; f=cdr(f))
	{
	    if (streq("name",get_c_string(car(car(f)))))
	    {
		has_name = TRUE;
		si_lyric->set_name(get_c_string(car(cdr(car(f)))));
	    }
	    else
		si_lyric->set(get_c_string(car(car(f))),
			      get_c_string(car(cdr(car(f)))));
	}
	if (!has_name)
	{
	    cerr << "add_lyric: lyric has description but no name" << endl;
	    cerr << "  " << siod_sprint(lyric) << endl;
	    EST_error("...");
	}
    }
    else // just the name
	si_lyric = add_lyric(u,get_c_string(lyric));

    return si_lyric;
}

